"use strict";
const entities = require('entities');
const xpath = require('xpath');
const path = require('path');
const moment = require('moment-timezone');
const dom = require('xmldom').DOMParser;
const Parser = require('./Parser.js');

module.exports = class ParserXML extends Parser{
    
    constructor(config, key) {
                
        super();
        
        this.config = config;
        this.key = key;
        
    }

    /**
     *
     ** @param doc - XML document
     *
     * return raw nowplaying item
     *
     */
    parseNowOnAirXML(doc, filename) {

        this.doc = doc;
        this.filename = filename;
        this.globalDate = this.getGlobalDate();

        var itemNodeAsNode = xpath.select1(this.config.items_xpath, doc);
        //node has to be parsed as DOM object, not Node object
        var itemNode = new dom().parseFromString(itemNodeAsNode.toString());

        if (itemNode) {
            var item = this.parseItemXML(itemNode);
        } else {
            throw Error("No Item found in document: ".this.config.items_xpath)
        }

        if (typeof item['start'] == "undefined") {
            throw Error("Now Playing item is missing field: start");
        }

        return item;
    }

    parsePlayListXML(doc, filename) {
        this.doc = doc;
        this.filename = filename;
        this.globalDate = this.getGlobalDate();

        return this.parseBlockXML(doc);
    }

    /**
     *
     * @param doc - XML document
     *
     * return block object with starttime and raw items
     *
     */
    parseBlockXML(doc, filename) {

        var itemNodesAsNode = xpath.select(this.config.items_xpath, doc);
        var items = [];

        for (var i=0; i < itemNodesAsNode.length; i++) {
            //node has to be parsed as DOM object, not Node object
            var itemNode = new dom().parseFromString(itemNodesAsNode[i].toString());
            try {
                let item = this.parseItemXML(itemNode);
                if (item) {
                    items.push(item);
                }
            } catch (e) {
                this.emitWarning('Cannot parse an item('+(i+1)+'), skipping: '+e.message);
            }
        }

        if (items.length < 1) {
            throw Error("Unable to parse playlist, no valid items found!");
        }
        var date = false;

        if (!this.globalDate) {
            //if global date is not set, get hour from first item
            if (typeof items[0].start == "undefined") {
                throw Error("Unable to parse playlist, no valid global date found and no start in first item.");
            }

            let start = moment(items[0].start);
            date = moment(start.format("YYYY-MM-DD HH")+":00:00");
        } else {
            date = this.globalDate
        }

        var output = {'start' : date.format(), 'items': items};
        
        if (this.config.append_items == true) {
            output.append = true;
        }

        return output;

    }

    /**
     *
     * Parse multible blocks divided by parent element, return array with block objects
     * 
     * @param doc - XML document
     * 
     */
    parseMultiBlockXMLByParent(doc, filename) {

        if (typeof this.config.blocks_xpath == "undefined") {
            throw Error("Config has multiblock by parent element, but no xpath found for blocks!")
        }
        var blocks = [];
        this.doc = doc;
        this.filename = filename;
        this.globalDate = this.getGlobalDate();

        var blockNodesAsNode = xpath.select(this.config.blocks_xpath, doc);
        for (var i = 0; i < blockNodesAsNode.length; i++) {
            //node has to be parsed as DOM object, not Node object
            var blockNode = new dom().parseFromString(blockNodesAsNode[i].toString());
            try {
                let block = this.parseBlockXML(blockNode, filename);
                blocks.push(block);
            } catch (e) {
                //put a empty block, because we recount the hours afeter
                blocks.push({'start': false, 'items': []});
                this.emitWarning('Cannot parse an block ('+(i+1)+'), skipping: ' + e.message);
            }
        }

        if (blocks.length < 1) {
            throw Error("Unable to parse playlist, no valid blocks found!");
        }

        if (!blocks[0].start) {
            throw Error("Unable to parse playlist, no valid global date or date in items found!");
        }

        if (this.config.append_items == true) {
            for(var i in blocks) {
                blocks[i].append = true;
            }
        }
        
        if (blocks.length == 1) {
            return blocks;
        }

        //fix block dates. if there's a global date, all blocks will have same start hour. Fix it by assuming all hours are 
        if (blocks[0].date == blocks[1].date) {
            for (var i = 1; i < blocks.length; i++) {
                blocks[i].start = moment(blocks[i].start).add(i, 'hours').format();
            }
        }

        return blocks;
    }

    /**
     * 
     * Parse multible blocks divided by specific key, return array with block objects
     * 
     * @param doc
     * @param filename
     * @returns {Array}
     */
    parseMultiBlockXMLByKey(doc, filename){

        if (typeof this.config.blocks_key == "undefined") {
            throw Error("Config has multiblock by key field, but no key found for blocks!")
        }

        this.doc = doc;
        this.filename = filename;
        this.globalDate = this.getGlobalDate();

        //first parse all items just as normal
        var blockKey = this.config.blocks_key;
        var parsedBlock = this.parseBlockXML(doc, filename);
        var allItems = parsedBlock.items;

        if (allItems.length < 1) {
            throw Error("Unable to parse playlist, no valid items found!");
        }

        var blocks = [];
        var blockItems = {};

        for (var i in allItems) {
            if (typeof allItems[i][blockKey] == "undefined") {
                throw Error("Config has multiblock by key field, but an item is missing field ".blockKey);
            }
            var currentBlock = allItems[i][blockKey];
            if (typeof blockItems[currentBlock] == "undefined") {
                blockItems[currentBlock] = [];
            }
            blockItems[currentBlock].push(allItems[i]);
        }
        var blockCount = 0;

        for (var key in blockItems) {
            var blockMoment = false;
            if (/^\d{1,2}$/.test(key) && parseInt(key) < 24) {
                //pretty standard is that the blockkey is an integer which than is the hour 
                //try this first, else moment might recognize it as a month
                blockMoment = this.parseDateTime(key+":00", "H:m");
            }
            if (!blockMoment || !blockMoment.isValid()) {
                //check if the blockKey has a date in itself, that would clarify things
                blockMoment = this.parseDateTime(key, true);
            }
            if ((!blockMoment || !blockMoment.isValid()) && this.globalDate) {
                //didn't work? don't give up, try again with a date in front
                blockMoment = this.parseDateTime(this.globalDate.format("YYYY-MM-DD ")+key, true);
            }
            if (blockMoment && blockMoment.isValid()) {
                //yes blockKey contains a date, but enforce it with global date if it's available
                let date = false;
                if (this.globalDate) {
                    date = this.parseDateTime(this.globalDate.format("YYYY-MM-DD") + ' ' + blockMoment.format("HH:mm:ss"), true);
                } else {
                    date = blockMoment;
                }

                blocks.push({'start': date.format(), 'items': blockItems[key]});
                continue;
            }
            //maybe item has start, then we know the hour as well!
            if (typeof blockItems[key][0].start != "undefined") {
                blocks.push({'start': blockItems[key][0].start.startOf('hour').format(), 'items': blockItems[key]});
                continue;
            }
            //no date in the block key, no starttimes in items, use the global date and lets count the blocks ourselves then.
            blocks.push({'start': moment(parsedBlock.date).add(blockCount, 'hours').format(), 'items': blockItems[key]});
            blockCount++;
        }

        if (this.config.append_items == true) {
            for(var i in blocks) {
                blocks[i].append = true;
            }
        }

        return blocks;
    }


    /**
     * 
     * Parse a single XML item
     * 
     * @param itemNode
     * @returns {*}
     */
    parseItemXML(itemNode) {

        let itemData = {};

        //first identify the model type and get the config for this type
        var currentModelType = this.getCurrentModelTypeConfigXML(itemNode);

        if (!currentModelType) {
            throw Error("Cannot parse item, no matching ModelType found!");
        }

        //Fill the item with all the data from the XML, according to the config of the ModeLtype
        for (let dataKey in currentModelType.data_field_xpaths) {
            let node = xpath.select1(currentModelType.data_field_xpaths[dataKey], itemNode);
            if (!node) {
                this.emitWarning('Invalid node with selector: ' + currentModelType.data_field_xpaths[dataKey] +', '+dataKey);
            } else {
                itemData[dataKey] = this.getNodeValue(node);
            }
        }

        //add the model_type_id from the config
        itemData['model_type_id'] = currentModelType.id;

        //finalize the item general stuff
        try {
            return this.processItem(itemData, currentModelType);
        } catch (e){
            throw Error(e);
        }

    }

    
    /**
     *  Extract the modeltype out of an XML Item
     */
    getCurrentModelTypeConfigXML(itemNode) {
        for (let i in this.config.items) {
            let modelType = this.config.items[i];
            if (typeof modelType.data_field_xpaths[modelType.model_type_field] != "undefined") {
                let node = xpath.select1(modelType.data_field_xpaths[modelType.model_type_field], itemNode);
                if (node) {
                    if (this.matchModelType(this.getNodeValue(node), modelType)) {
                        return modelType
                    }
                }
            }
        }
        return false;
    }


    /**
     * get a value from a specific node extracted with Xpath
     *
     * @param {Object} node
     * @returns {String}
     */
    getNodeValue(node) {
        if (!node) throw Error("Cannot read node propery, something went wrong parsing the XML.")
        switch (node.nodeType) {
            case 1:
                return node.nodeName; // element
            case 2:
                return entities.decodeXML(node.value); // attribute
            case 3:
                return entities.decodeXML(node.toString()); // text
            case 4:
                return entities.decodeXML(node.data);
            default:
                return null;
        }
    }
    

    
}